#! /usr/bin/env python

"""
ACH Thumbstick Click RPi Demo V1
2014-08-20 10.40pm

Created by Andrew Hazelden
Email: andrew@andrewhazelden.com
Blog: http://www.andrewhazelden.com
Twitter: https://www.twitter.com/andrewhazelden

Raspberry PI Interfacing Notes

The Thumbstick Click board is connected to a Raspberry Pi using the MikroE Pi Click Shield. This shield connects to the Raspberry PI GPIO header pins and lets you attach MikroBus format Click boards.

If you don't have a Pi Click Shield, you can get one from the MikroElektronika Webstore, or from Digikey, or Farnell.

The Thumbstick Click board uses an SPI based MCP3204 module for reading the joystick axes. The ADC0 channel is used for the y-axis and the ADC1 channel is used for the x-axis. The board is powered on 3.3 Volts from the GPIO header on the Raspberry PI.

The joystick "center click" button is read using the GPIO input 17 on the Raspberry Pi and has an inverted logic state where the button clicked=0, and the button unclicked=1.
If you take the rubber joystick cap off the thumbstick you can see the mechanical push button that responds to the clicking motion.

Admin Privileges Required
Scripts that use the GPIO and SPI hardware interfaces need to be run with sudo since GPIO reading requires admin privileges on Linux.

That means you need to run the thumbstick.py demo python script using:
sudo python thumbstick.py

Driver Setup and Installation

Step 1. Let's enable SPI on the Pi using the following terminal command: ( Tip from the projects.drogon.net blog)

sudo modprobe spi_bcm2708
sudo chown `id -u`.`id -g` /dev/spidev0.*

Then check the SPI module is active with:
lsmod

You should now see the following module listed:
spi_bcm2708

Step 2. Install webiopi so it is easy to use SPI inputs from python scripts.

You can download the free webiopi library from:
http://sourceforge.net/projects/webiopi/files

Then you need to Un-gzip the file "WebIOPi-0.7.0.tar.gz". You can do that by right clicking on the file and using the "Extract Here" tool on Raspbian.

Run the webiopi setup program with:
cd WebIOPi-0.7.0
sudo ./setup.sh

It is a good idea to setup a webiopi username and password for the integrated server module:
sudo webiopi-passwd

Note: The webiopi generated password file is stored in:
/etc/webiopi/passwd

Step 3. Edit the webiopi config file with:
sudo gedit /etc/webiopi/config

Scroll down to line 96 in the webiopi "config" file and add the following code:
#MikroE Thumbstick IO on a Raspberry Pi Click Shield
adc = MCP3204

This tells the webiopi software that the Thumbstick Click board uses the MCP3204 SPI based chip for ADC input.

Step 4. Run the thumbstick.py demo script using:
sudo python thumbstick.py

If the script runs correctly you will be presented with the following output in the terminal:

You can quit the python script by pressing the Control+D hotkey.

Using the Webiopi WebGUI

As an optional step, you can start the webiopi server module for http based SPI and GPIO access.

Note: You need to set up the webiopi password in step 3 if you want to use the webserver module.

The webiopi server module creates an http hosted webgui page that updates on your pi. 

After you start the server module you can access it using either:
http://localhost:8000
or
http://<write your raspberry pi ip address here>:8000

You can start webiopi's webserver module with ONE of the following options:
Start webiopi from a terminal session with:
webiopi -d -c /etc/webiopi/config

To start webiopi in the background:
sudo /etc/init.d/webiopi start

To start webiopi at boot:
sudo update-rc.d webiopi defaults

Webiopi GPIO Header

You can use the webGUI to read the current GPIO pin state from a web browser. This is interesting as you can see the input and output mapping and see the binary state of each of the GPIO pins.

Webiopi Device Monitor

You can also watch the live values on the Thumbstick's 2 ADC channels (Joystick X & Y Axis) using the webiopi "Device Monitor" webpage.

"""

print 'ACH Thumbstick Click Demo'
print 'Version 1.0 - 2014-08-20'
print '-------------------------'
print 'Press Control+D to Quit'

#Used for sleep command
from time import sleep

# Set up the MCP320 ADC based 2 Channel joystick X/Y axis
from webiopi import deviceInstance
from webiopi.devices.analog.mcp3x0x import MCP3204

# Set up the GPIO input 17 joystick button reading
import RPi.GPIO as GPIO

# Use GPIO Numbers
GPIO.setmode(GPIO.BCM)

# Setup the GPIO Channels
# GPIO input 17 is the center joystick button
GPIO.setup(17, GPIO.IN)

# Init the MCP3204 ADC input chip on the thumbstick click board
mcp0 = MCP3204()

# Global Vars
thumbstick_total_adc_channels = mcp0.analogCount()
thumbstick_max_range = mcp0.analogMaximum()
thumbstick_button = 0;

print '[Thumbstick X/Y Axis Range] 0 to ' + str(thumbstick_max_range)

def GetADC(adc_channel):
  # Read one of the joystick axis
  value = mcp0.analogRead(adc_channel)
  return value
  
  
def GetButton(gpio_channel):
  # Read Input from GPIO17
  # The native thumbstick button has the logic state inverted where button clicked = 0, button unclicked = 1. We apply the 'not' command to make the button input easier to understand.
  
  button = not GPIO.input(gpio_channel)
  return button


def ReadThumbstick():
 # Read the thumbstick button input from GPIO17
  button = GetButton(17)
  button_text = ''

  if(button):
    button_text = 'Clicked'
  else:
    button_text = 'Not Clicked'
   
  # Read the joystick values
  y_axis = GetADC(0)
  x_axis = GetADC(1)

  # Display the results
  print ('[X]' + str(y_axis) + ' [Y]' + str(x_axis) + ' [Button] ' + button_text)

  return button
  

# Run the main code
while True:
  # Read the Thumbstick values
  thumbstick_button = ReadThumbstick()
  
  # Wait half a second between updates to slow down the scroll rate
  sleep(0.5)



